package dev.pfaff.altargui.pathfinder.fontkit;

import dev.pfaff.altargui.pathfinder.RectI;
import org.jspecify.annotations.Nullable;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;

import static dev.pfaff.altargui.pathfinder.Pathfinder.*;
import static java.lang.foreign.ValueLayout.JAVA_FLOAT;
import static java.lang.foreign.ValueLayout.JAVA_INT;

public record FKFontRef(MemorySegment ptr) {
	@Nullable
	public Integer glyphForCodepoint(int codepoint) {
		try {
			var result = (long) MH_FKFontGlyphForCodepoint.invokeExact(ptr(), codepoint);
			if ((result >>> 32) != 0) {
				return (int) result;
			} else {
				return null;
			}
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void rasterizeCodepoint(MemorySegment canvasPixelBuffer,
								   int canvasWidth, int canvasHeight,
								   FKCanvasFormat canvasFormat,
								   int glyphId,
								   float pointSize,
								   float translationX, float translationY,
								   FKCanvasRasterizationOptions rasterizationOptions) {
		try (var arena = Arena.ofConfined()) {
			var canvasSize = arena.allocateFrom(JAVA_INT, canvasWidth, canvasHeight);
			var translation = arena.allocateFrom(JAVA_FLOAT, translationX, translationY);
			var result = (int) MH_FKFontRasterizeGlyph.invokeExact(ptr(),
					canvasPixelBuffer,
					canvasPixelBuffer.byteSize(),
					canvasSize,
					canvasFormat.value(),
					glyphId,
					pointSize,
					translation,
					rasterizationOptions.value());
			if (result != 0) {
				throw new RuntimeException();
			}
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public RectI rasterBounds(int glyphId, float pointSize, FKCanvasRasterizationOptions options) {
		try (var arena = Arena.ofConfined()) {
			var dst = arena.allocate(ML_PFRectI);
			var result = (int) MH_FKFontRasterBounds.invokeExact(ptr(), glyphId, pointSize, options.value(), dst);
			if (result != 0) throw new RuntimeException();
			var originX = dst.getAtIndex(JAVA_INT, 0);
			var originY = dst.getAtIndex(JAVA_INT, 1);
			var lowerRightX = dst.getAtIndex(JAVA_INT, 2);
			var lowerRightY = dst.getAtIndex(JAVA_INT, 3);
			return new RectI(originX, originY, lowerRightX - originX, lowerRightY - originY);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
