package dev.pfaff.altargui.texture;

import dev.pfaff.altargui.types.Tex2d;

import java.lang.foreign.MemorySegment;

public final class TextureHandle {
	/**
	 * A zero-length memory segment that keeps the texture atlas region alive. The cleanup action will decrement the
	 * ref-count in the allocator.
	 */
	@SuppressWarnings({"FieldCanBeLocal", "unused"})
	private final MemorySegment handle;
	public final int layer;
	/**
	 * The base U of the texture in the atlas. Normalized.
	 */
	public final char u;
	/**
	 * The base U of the texture in the atlas. Normalized.
	 */
	public final char v;
	/**
	 * The width of the texture divided by the width of the atlas, normalized to {@code [0, 65536]}.
	 */
	private final int textureWidthAtlas;
	/**
	 * The height of the texture divided by the height of the atlas, normalized to {@code [0, 65536]}.
	 */
	private final int textureHeightAtlas;

	TextureHandle(MemorySegment handle,
				  int layer,
				  char u,
				  char v,
				  TextureAtlasScale atlasScale,
				  int textureWidth,
				  int textureHeight) {
		this.handle = handle;
		this.layer = layer;
		this.u = u;
		this.v = v;
		this.textureWidthAtlas = atlasScale.pixelsToAtlasX(textureWidth);
		this.textureHeightAtlas = atlasScale.pixelsToAtlasY(textureHeight);
	}

	public static TextureHandle unmanaged(int layer,
										  char u,
										  char v,
										  TextureAtlasScale atlasScale,
										  int textureWidth,
										  int textureHeight) {
		return new TextureHandle(MemorySegment.NULL, layer, u, v, atlasScale, textureWidth, textureHeight);
	}

	public int texToAtlasX(float x) {
		Tex2d.checkTexCoordBounds(x);
		// TODO: round or not?
		return this.u + Math.round(this.textureWidthAtlas * x);
	}

	public int texToAtlasY(float y) {
		Tex2d.checkTexCoordBounds(y);
		return this.v + Math.round(this.textureHeightAtlas * y);
	}
}
